#!/bin/bash

#set -x
 
# 配置参数
PERSISTENT_HANDLE="0x81010002"  # 固化的持久句柄
SPLIT_SIZE="256"                # 分块大小

usage() {
    echo "Usage: $0 -e <input_file>    # 加密文件"
    echo "       $0 -d <encrypted_file> # 解密文件"
    exit 1
}

# 加密函数
encrypt() {
    local input_file="$1"
    local output_file="${input_file}.enc"
    
    # 拆分文件
    split -b "$SPLIT_SIZE" -d "$input_file" "enc_split_"
    if [ $? -ne 0 ]; then
        echo "文件拆分失败！"
        exit 1
    fi

    # 加密分块（直接使用持久句柄）
    for part in enc_split_*; do
        tpm2_encryptdecrypt -c "$PERSISTENT_HANDLE" -o "${part}.enc" "$part"
        if [ $? -ne 0 ]; then
            echo "加密失败：$part"
            exit 1
        fi
        rm "$part"  # 删除原始分块
    done

    # 合并加密块
    cat enc_split_*.enc > "$output_file"
    rm enc_split_*.enc  # 清理中间文件
    echo "加密完成：$output_file"
}

# 解密函数
decrypt() {
    local input_file="$1"
    local output_file="${input_file%.enc}"

    # 拆分加密文件
    split -b "$SPLIT_SIZE" -d "$input_file" "dec_split_"
    if [ $? -ne 0 ]; then
        echo "文件拆分失败！"
        exit 1
    fi

    # 解密分块（直接使用持久句柄）
    for part in dec_split_*; do
        tpm2_encryptdecrypt -c "$PERSISTENT_HANDLE" -d -o "${part}.dec" "$part"
        if [ $? -ne 0 ]; then
            echo "解密失败：$part"
            exit 1
        fi
        rm "$part"  # 删除原始加密分块
    done

    # 合并解密块
    cat dec_split_*.dec > "$output_file"
    rm dec_split_*.dec  # 清理中间文件
    echo "解密完成：$output_file"
}

# 主流程
while getopts "e:d:" opt; do
    case $opt in
        e)
            encrypt "$OPTARG"
            exit $?
            ;;
        d)
            decrypt "$OPTARG"
            exit $?
            ;;
        *)
            usage
            ;;
    esac
done

usage
